#!/usr/bin/env python
""" Copyright 2020 Zhao HG

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""
with open('UnicodeData.txt', 'r') as reader:
    categories = [line.strip().split(';')[2] for line in reader]
    base_categories = [cat[0] for cat in categories]

with open('include/unicode_char.h', 'a') as writer:
    writer.write('/** The types of general category. */\n')
    writer.write('enum class GeneralCategory {')
    for i, category in enumerate(sorted(list(set(categories)))):
        if i == 0:
            writer.write('\n    ')
        elif i % 10 == 0:
            writer.write(',\n    ')
        else:
            writer.write(', ')
        writer.write(category)
    writer.write('\n};\n\n')
    writer.write('/** The types of base general category. */\n')
    writer.write('enum class BaseGeneralCategory {')
    for i, category in enumerate(sorted(list(set(base_categories)))):
        if i == 0:
            writer.write('\n    ')
        elif i % 10 == 0:
            writer.write(',\n    ')
        else:
            writer.write(', ')
        writer.write(category)
    writer.write('\n};\n\n')

    writer.write('/** Outputs the general category with its name. */\n')
    writer.write('std::ostream& operator<<(std::ostream&, GeneralCategory);\n')
    writer.write('/** Outputs the base general category with its name. */\n')
    writer.write('std::ostream& operator<<(std::ostream&, BaseGeneralCategory);\n\n')

with open('include/unicode_data.h', 'a') as writer:
    writer.write('/** The general category data. */\n')
    writer.write('extern const GeneralCategory GENERAL_CATEGORY[];\n')
    writer.write('/** The base general category data. */\n')
    writer.write('extern const BaseGeneralCategory BASE_GENERAL_CATEGORY[];\n\n')

with open('src/general_category.cpp', 'w') as writer:
    with open('copyright.txt', 'r') as reader:
        writer.write(reader.read())

    writer.write('#include "unicode_data.h"\n\n')
    writer.write('namespace unicode {\n\n')

    writer.write('std::ostream& operator<<(std::ostream& os, GeneralCategory c) {\n')
    writer.write('    switch (c) {\n')
    for category in sorted(list(set(categories))):
        writer.write('    case GeneralCategory::{}: os << "{}"; break;\n'.format(category, category))
    writer.write('    }\n')
    writer.write('    return os;\n')
    writer.write('}\n\n')

    writer.write('std::ostream& operator<<(std::ostream& os, BaseGeneralCategory c) {\n')
    writer.write('    switch (c) {\n')
    for category in sorted(list(set(base_categories))):
        writer.write('    case BaseGeneralCategory::{}: os << "{}"; break;\n'.format(category, category))
    writer.write('    }\n')
    writer.write('    return os;\n')
    writer.write('}\n\n')

    writer.write('\nconst GeneralCategory GENERAL_CATEGORY[] = {')
    for i, category in enumerate(categories):
        if i == 0:
            writer.write('\n    ')
        elif i % 5 == 0:
            writer.write(',\n    ')
        else:
            writer.write(', ')
        writer.write('GeneralCategory::%s' % category)
    writer.write('\n};\n\n')

    writer.write('\nconst BaseGeneralCategory BASE_GENERAL_CATEGORY[] = {')
    for i, category in enumerate(base_categories):
        if i == 0:
            writer.write('\n    ')
        elif i % 4 == 0:
            writer.write(',\n    ')
        else:
            writer.write(', ')
        writer.write('BaseGeneralCategory::%s' % category)
    writer.write('\n};\n\n')

    writer.write('}  // namespace unicode\n')

with open('tests/test_general_category_gen.cpp', 'w') as writer:
    with open('copyright.txt', 'r') as reader:
        writer.write(reader.read())

    writer.write('#include "test.h"\n')
    writer.write('#include "unicode_char.h"\n\n')
    writer.write('namespace test {\n\n')

    writer.write('class GeneralCategoryGenTest : public UnitTest {};\n\n')

    writer.write('__TEST_U(GeneralCategoryGenTest, test_cats) {\n')
    appeared = set()
    with open('UnicodeData.txt', 'r') as reader:
        for line in reader:
            parts = line.strip().split(';')
            category = parts[2]
            if category not in appeared:
                appeared.add(category)
                writer.write('    __ASSERT_EQ(unicode::GeneralCategory::{}, unicode::getGeneralCategory({}));\n'.format(
                    category, '0x' + parts[0]
                ))
    writer.write('}\n\n')

    writer.write('__TEST_U(GeneralCategoryGenTest, test_base_cats) {\n')
    appeared = set()
    with open('UnicodeData.txt', 'r') as reader:
        for line in reader:
            parts = line.strip().split(';')
            category = parts[2][0]
            if category not in appeared:
                appeared.add(category)
                writer.write('    __ASSERT_EQ(unicode::BaseGeneralCategory::{}, '
                             'unicode::getBaseGeneralCategory({}));\n'.format(category, '0x' + parts[0]))
    writer.write('}\n\n')

    writer.write('}  // namespace test\n')



import os
import random
from unittest import TestCase
import mixal


class TestSample(TestCase):

    def setUp(self) -> None:
        self.computer = mixal.Computer()

    def load_codes(self, name):
        current_path = os.path.dirname(os.path.abspath(__file__))
        file_path = os.path.join(current_path, name + '.mix')
        with open(file_path) as reader:
            self.computer.loadCodes(reader.read())

    def test_16_set_all_zeros_a(self):
        self.load_codes('16_set_all_zeros_a')
        for i in range(100):
            self.computer.memoryAt(i).set(random.randint(1, 100))
        self.computer.executeUntilHalt()
        for i in range(100):
            self.assertEqual(0, self.computer.memoryAt(i).value(), i)
        print('Cost:', self.computer.elapsed())

    def test_17_set_all_zeros_with_n_a(self):
        max_case_num = 10
        for cases in range(max_case_num):
            self.load_codes('17_set_all_zeros_with_n_a')
            n = 0
            if cases == max_case_num - 1:
                n = 2999
            elif cases > 0:
                n = random.randint(0, 3000)
            for i in range(3000):
                self.computer.memoryAt(i).set(random.randint(1, 100))
            self.computer.rI2.set(n)
            self.computer.executeUntilHalt()
            for i in range(n + 1):
                self.assertEqual(0, self.computer.memoryAt(i).value(), i)
            for i in range(n + 1, 3000):
                self.assertLess(0, self.computer.memoryAt(i).value(), i)
        print('Cost:', self.computer.elapsed())

    def test_17_set_all_zeros_with_n_b(self):
        max_case_num = 10
        for cases in range(max_case_num):
            self.load_codes('17_set_all_zeros_with_n_b')
            n = 0
            if cases == max_case_num - 1:
                n = 2999
            elif cases > 0:
                n = random.randint(0, 3000)
            for i in range(3000):
                self.computer.memoryAt(i).set(random.randint(1, 100))
            self.computer.rI2.set(n)
            self.computer.executeUntilHalt()
            for i in range(n + 1):
                self.assertEqual(0, self.computer.memoryAt(i).value(), i)
            for i in range(n + 1, 3000):
                self.assertLess(0, self.computer.memoryAt(i).value(), i)
        print('Cost:', self.computer.elapsed())

    def test_17_set_all_zeros_with_n_b_official(self):
        self.load_codes('17_set_all_zeros_with_n_b_official')
        n = 2999
        for i in range(3000):
            self.computer.memoryAt(i).set(random.randint(1, 100))
        self.computer.rI2.set(n)
        self.computer.executeUntilHalt()
        for i in range(n + 1):
            self.assertEqual(0, self.computer.memoryAt(i).value(), i)
        for i in range(n + 1, 3000):
            self.assertLess(0, self.computer.memoryAt(i).value(), i)
        print('Cost:', self.computer.elapsed())

    def test_18_number_one(self):
        self.load_codes('18_number_one')
        self.computer.executeUntilHalt()
        self.assertEqual(42, self.computer.elapsed())

    def test_20_set_all_halt(self):
        self.load_codes('20_set_all_halt')
        self.computer.executeUntilHalt()
        for i in range(4000):
            self.assertEqual(133, self.computer.memoryAt(i).value())
        print('Exit:', self.computer.line())
        print('Cost:', self.computer.elapsed())

    def test_21_set_rj(self):
        for _ in range(10):
            target = random.randint(1, 3000)
            randoms = [random.randint(0, 4096) for _ in range(3000)]
            self.load_codes('21_set_rj')
            self.computer.rI4.set(target)
            for i in range(3000):
                self.computer.memoryAt(i).set(randoms[i])
            self.computer.executeUntilHalt()
            self.assertEqual(target, self.computer.rJ.value())
            for i in range(3000):
                self.assertEqual(randoms[i], self.computer.memoryAt(i).value(), (target, i))
        print('Cost:', self.computer.elapsed())

    def test_22_x_13_official(self):
        for x in range(-4, 5):
            self.load_codes('22_x_13_official')
            self.computer.memoryAt(2000).set(x)
            self.computer.executeUntilHalt()
            self.assertEqual(x ** 13, self.computer.rA.value(), x)
        print('Cost:', self.computer.elapsed())

    def test_23_reflect(self):
        self.load_codes('23_reflect')
        self.computer.memoryAt(200).set(False, 1, 22, 33, 44, 5)
        self.computer.executeUntilHalt()
        self.assertEqual('+  5 44 33 22  1', self.computer.rA.getBytesString())
        print('Cost:', self.computer.elapsed())



import random
from unittest import TestCase
import mixal


class TestSample(TestCase):

    def setUp(self) -> None:
        self.computer = mixal.Computer()

    def test_sample(self):
        self.computer.loadCodes("""
X       EQU  1000
        ORIG 3000
MAXIMUM STJ  EXIT
INIT    ENT3 0,1
        JMP  CHANGEM
LOOP    CMPA X,3
        JGE  *+3
CHANGEM ENT2 0,3
        LDA  X,3
        DEC3 1
        J3P  LOOP
EXIT    JMP  *
        ORIG 3500
        HLT
        """)

        n, max_val = 100, 0
        self.computer.rI1.set(n)
        self.computer.rJ.set(3500)
        values = []
        for i in range(1001, 1001 + n):
            val = random.randint(0, 100000)
            self.computer.memoryAt(i).set(val)
            max_val = max(max_val, val)
            values.append(val)
        self.computer.executeUntilHalt()
        self.assertEqual(max_val, self.computer.rA.value(), str(values))

    def test_basic_io(self):
        card_reader_index = 16
        card_punch_index = 17

        self.computer.loadCodes(f"""
        ORIG 3000
        IN   100({card_reader_index})
LIN     JBUS LIN({card_reader_index})
        OUT  100({card_punch_index})
LOUT    JBUS LOUT({card_punch_index})
        """)
        self.computer.getDeviceWordAt(card_reader_index, 0).set('PRIME')
        self.computer.executeUntilHalt()
        print(self.computer.getDeviceWordAt(card_punch_index, 0).getCharacters())



import re
import sys

from src.generate_get_instruction import OPS

file_path = sys.argv[1]
ops = set(OPS)

with open(file_path) as reader:
    lines = []
    for line in reader:
        line = re.sub(r'^\d+', '', line).strip()
        if line.startswith('*') or len(line) == 0:
            lines.append(line)
            continue
        first, rest = line.split(' ', 1)
        if first in ops:
            loc, op = '', first
            rest = rest.split(' ', 1)
            address, comment = rest[0], ''
            if len(rest) > 1:
                comment = rest[1]
        else:
            loc = first
            rest = rest.split(' ', 2)
            op, address, comment = rest[0], rest[1], ''
            if len(rest) > 2:
                comment = rest[2]
        line = loc
        line += ' ' * (8 - len(line))
        line += op
        line += ' ' * (13 - len(line))
        line += address
        line += ' ' * (32 - len(line))
        line += comment
        line = line.rstrip()
        lines.append(line)

with open(file_path, 'w') as writer:
    for line in lines:
        writer.write(line + '\n')



import os
from unittest import TestCase
import mixal


class TestSample(TestCase):

    def setUp(self) -> None:
        self.computer = mixal.Computer()

    def load_codes(self, name):
        current_path = os.path.dirname(os.path.abspath(__file__))
        file_path = os.path.join(current_path, name + '.mix')
        with open(file_path) as reader:
            self.computer.loadCodes(reader.read())

    def init_input_device(self, device, text):
        for i in range(0, len(text), 5):
            self.computer.getDeviceWordAt(device, i // 5).set(text[i:i + 5])

    def test_a_multiply_permutations_in_cycle_form(self):
        self.load_codes('a_multiply_permutations_in_cycle_form')
        input_text = '    (  A    C    F    G  )        (  B    C    D  )        (  A    E    D  )    ' \
                     '    (  F    A    D    E  )        (  B    G    F    A    E  )                  ='
        self.init_input_device(16, input_text)
        self.computer.executeUntilHalt()
        output = ''
        for i in range(48, 58):
            output += self.computer.getDeviceWordAt(18, i).getCharacters()
        self.assertEqual('    (  A    D    G  )        (  C    E    B  )    ', output)
        print('Cost:', self.computer.elapsed())

    def test_b_same_effects_as_a(self):
        self.load_codes('b_same_effects_as_a')
        input_text = '    (  A    C    F    G  )        (  B    C    D  )        (  A    E    D  )    ' \
                     '    (  F    A    D    E  )        (  B    G    F    A    E  )                  ='
        self.init_input_device(16, input_text)
        self.computer.executeUntilHalt()
        output = ''
        for i in range(48, 58):
            output += self.computer.getDeviceWordAt(18, i).getCharacters()
        self.assertEqual('    (  C    E    B  )        (  D    G    A  )    ', output)
        print('Cost:', self.computer.elapsed())

    def test_i_inverse_in_space(self):
        self.load_codes('i_inverse_in_space')
        self.computer.executeUntilHalt()
        expected = [3, 2, 6, 5, 4, 1]
        for i, val in enumerate(expected):
            self.assertEqual(val, self.computer.memoryAt(1001 + i).value())
        print('Cost:', self.computer.elapsed())

    def test_j_analogous_to_program_i(self):
        self.load_codes('j_analogous_to_program_i')
        self.computer.executeUntilHalt()
        expected = [3, 2, 6, 5, 4, 1]
        for i, val in enumerate(expected):
            self.assertEqual(val, self.computer.memoryAt(1001 + i).value())
        print('Cost:', self.computer.elapsed())
